==> pmvvm-2.0.1.tar.gz: lib/mvvm_builder.widget.dart <==
import 'package:flutter/widgets.dart';
import 'package:flutter/services.dart';
import 'package:pmvvm/pmvvm.dart';
import 'view_model.dart';

class MVVM<T extends ChangeNotifier> extends StatefulWidget {
  /// A builder function for the View widget, it also has access
  /// to the viewmodel.
  final Widget Function(BuildContext, T) view;

  /// A builder function for the viewmodel of the view.
  final T viewModel;

  /// To dispose the viewmodel when the provider is removed from the
  /// widget tree.

  /// default's to [true]
  final bool disposeVM;

  /// Init the viewmodel only once
  final bool initOnce;

  const MVVM({
    Key? key,
    required this.view,
    required this.viewModel,
    this.disposeVM = true,
    this.initOnce = false,
  }) : super(key: key);

  @override
  _MVVMState<T> createState() => _MVVMState<T>();
}

class _MVVMState<T extends ChangeNotifier> extends State<MVVM<T>> {
  late T _vm;
  bool _initialised = false;

  @override
  void initState() {
    super.initState();
    _vm = widget.viewModel;

    SystemChannels.lifecycle.setMessageHandler((msg) async {
      if (msg == AppLifecycleState.resumed.toString()) {
        (_vm as ViewModel).onResume();
      } else if (msg == AppLifecycleState.inactive.toString()) {
        (_vm as ViewModel).onInactive();
      } else if (msg == AppLifecycleState.paused.toString()) {
        (_vm as ViewModel).onPause();
      } else if (msg == AppLifecycleState.detached.toString()) {
        (_vm as ViewModel).onDetach();
      }

      return '';
    });
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    if (identical(_vm, widget.viewModel)) {
      _vm = widget.viewModel;
    }

    (_vm as ViewModel).context = this.context;

    if (widget.initOnce && !_initialised) {
      (_vm as ViewModel).init();
      _initialised = true;
    } else if (!widget.initOnce) {
      (_vm as ViewModel).init();
    }
  }

  @override
  Widget build(BuildContext context) {
    (_vm as ViewModel).onBuild();

    if (!widget.disposeVM) {
      return ChangeNotifierProvider.value(
        value: _vm,
        child: widget.view(context, _vm),
      );
    }

    return ChangeNotifierProvider(
      create: (context) => _vm,
      child: widget.view(context, _vm),
    );
  }
}

==> pmvvm-2.2.0.tar.gz: lib/mvvm_builder.widget.dart <==
import 'package:flutter/widgets.dart';
import 'package:pmvvm/pmvvm.dart';
import 'view_model.dart';

/// MVVM builder widget
class MVVM<T extends ViewModel> extends StatefulWidget {
  /// A builder function for the View widget, it also has access
  /// to the viewmodel.
  final Widget Function(BuildContext, T) view;

  /// A builder function for the viewmodel of the view.
  final T viewModel;

  /// To dispose the viewmodel when the provider is removed from the
  /// widget tree.

  /// default's to [true]
  final bool disposeVM;

  /// Init the viewmodel only once
  final bool initOnce;

  const MVVM({
    Key? key,
    required this.view,
    required this.viewModel,
    this.disposeVM = true,
    this.initOnce = false,
  }) : super(key: key);

  @override
  _MVVMState<T> createState() => _MVVMState<T>();
}

class _MVVMState<T extends ViewModel> extends State<MVVM<T>>
    with WidgetsBindingObserver {
  late T _vm;
  bool _initialised = false;

  @override
  void initState() {
    super.initState();
    _vm = widget.viewModel;

    WidgetsBinding.instance?.addObserver(this);
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.resumed) {
      _vm.onResume();
    } else if (state == AppLifecycleState.inactive) {
      _vm.onInactive();
    } else if (state == AppLifecycleState.paused) {
      _vm.onPause();
    } else if (state == AppLifecycleState.detached) {
      _vm.onDetach();
    }
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    if (identical(_vm, widget.viewModel)) {
      _vm = widget.viewModel;
    }

    _vm.context = context;

    if (widget.initOnce && !_initialised) {
      _vm.init();
      _initialised = true;
    } else if (!widget.initOnce) {
      _vm.init();
    }
  }

  @override
  void dispose() {
    WidgetsBinding.instance?.removeObserver(this);
    _vm.onDispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    _vm.onBuild();

    if (!widget.disposeVM) {
      return ChangeNotifierProvider.value(
        value: _vm,
        child: widget.view(context, _vm),
      );
    }

    return ChangeNotifierProvider(
      create: (context) => _vm,
      child: widget.view(context, _vm),
    );
  }
}
